Welcome to Sound Studio.

This is a graphical sound sample editor, with recording and playback facilities and a built-in graphical mixer, for Linux.

This is the second version, adopted for Tk4.0. To run this version a version of TCL 7.4 and Tk4.0 is required on your machine.

There is a version for TCL 7.3 and Tk 3.6 available. This version has no added features to that version, other than those due to the better facilities of Tk4.0 and the command line loading explained below.

Written by Paul D. Sharpe 1994-1995, Leeds, UK.
To decompress use:
			tar xvzf studio.tgz
This contains 
	1. executable TCL/tk script
	2. source C code for studio components
	3. icons for use in studio.
	4. the on-line help text
	5. the studio post-script manual (zipped).
	6. a compiled version of Sox (Linux patch) and manual page.
	7. the zipped-archive of the sox source.
To compile studio type;
			./install <Return>
in the directory containing the studio files. Then edit,if necessary, the path names in the file named studio.

To run studio, enter the command, studio, from a shell-tool running in X-windows.
It is also possible to load a file into studio directly from the commandline, by typing
  studio path/filename <Return>
This will load the file and play it. Thus it should be possible to configure a web browser to play sound files via. studio.

To uncompress the manual simply type;	 gunzip studio_man.ps.gz

For more details you can access the online documentation, providing wish is installed, by entering
 wish StudioHelp <return>
from within the directory containing the studio files.

Any problems? E-mail my supervisor at een6njb@sun.leeds.ac.uk he should be able to pass it on to me or answer simple queries.


Patch history:

	a: got rid of silly bug in init.tk (didn't manifest itself on
	   Paul's or my machine!).  Thanks to Jasom Adams.  7/11/96.
